@@ -27,6 +27,7 @@ zypper --non-interactive install --force-resolution --no-recommends \
 	docbook_5 \
 	docbook5-xsl-stylesheets \
 	e2fsprogs \
+	git-core \
 	gzip \
 	ibmswtpm2 \
 	keyutils-devel \
@@ -47,6 +48,8 @@ zypper --non-interactive install --force-resolution --no-recommends \
 
 zypper --non-interactive install --force-resolution --no-recommends \
 	gnutls openssl-engine-libp11 softhsm || true
+zypper --non-interactive install --force-resolution --no-recommends \
+	gnutls pkcs11-provider softhsm || true
 
 if [ -f /usr/lib/ibmtss/tpm_server -a ! -e /usr/local/bin/tpm_server ]; then
 	ln -s /usr/lib/ibmtss/tpm_server /usr/local/bin
