@@ -1,4 +1,4 @@
-#!/bin/sh -ex
+#!/bin/bash -ex
 
 # No need to run via sudo if we already have permissions.
 # Also, some distros do not have sudo configured for root:
@@ -9,13 +9,8 @@ else
 	SUDO=sudo
 fi
 
-version=1682
-
-wget --no-check-certificate https://sourceforge.net/projects/ibmswtpm2/files/ibmtpm${version}.tar.gz/download
-mkdir ibmtpm$version
-cd ibmtpm$version
-tar --no-same-owner -xvzf ../download
-cd src
-make -j$(nproc)
+git clone https://git.code.sf.net/p/ibmswtpm2/tpm2
+pushd tpm2/src 1>/dev/null || exit 1
+make -j"$(nproc)"
 $SUDO cp tpm_server /usr/local/bin/
-cd ../..
+popd 1>/dev/null
